@@ -1,17 +1,20 @@
 import datetime
 import gc
 import traceback
-from typing import Callable, List, Type, Union, Optional, Dict
+from typing import Callable, Dict, List, Optional, Type, Union
 
 import numpy as np
 from deap import tools
 from sklearn.metrics import mean_squared_error, roc_auc_score as roc_auc
 
-from fedot.api.api_utils.initial_assumptions import ApiInitialAssumptions
+from fedot.api.api_utils.assumptions.assumptions_builder import AssumptionsBuilder
 from fedot.api.api_utils.metrics import ApiMetrics
+from fedot.api.api_utils.presets import update_builder
+from fedot.api.time import ApiTime
 from fedot.core.composer.composer_builder import ComposerBuilder
 from fedot.core.composer.gp_composer.gp_composer import (PipelineComposerRequirements)
 from fedot.core.composer.gp_composer.specific_operators import boosting_mutation, parameter_change_mutation
+from fedot.core.constants import MINIMAL_SECONDS_FOR_TUNING, DEFAULT_TUNING_ITERATIONS_NUMBER
 from fedot.core.data.data import InputData
 from fedot.core.data.data_split import train_test_data_setup
 from fedot.core.data.multi_modal import MultiModalData
@@ -23,6 +26,7 @@ from fedot.core.optimisers.gp_comp.operators.mutation import MutationTypesEnum
 from fedot.core.optimisers.optimizer import GraphOptimiser, GraphOptimiserParameters
 from fedot.core.optimisers.utils.pareto import ParetoFront
 from fedot.core.pipelines.pipeline import Pipeline
+from fedot.core.repository.dataset_types import DataTypesEnum
 from fedot.core.repository.operation_types_repository import get_operations_for_task
 from fedot.core.repository.quality_metrics_repository import (MetricsRepository)
 from fedot.core.repository.tasks import Task, TaskTypesEnum
@@ -33,13 +37,15 @@ class ApiComposer:
 
     def __init__(self, problem: str):
         self.metrics = ApiMetrics(problem)
-        self.initial_assumptions = ApiInitialAssumptions()
         self.optimiser = EvoGraphOptimiser
         self.optimizer_external_parameters = None
         self.current_model = None
         self.best_models = None
         self.history = None
 
+        self.preset_name = None
+        self.timer = None
+
     def obtain_metric(self, task: Task, composer_metric: Union[str, Callable]):
         # the choice of the metric for the pipeline quality assessment during composition
         if composer_metric is None:
@@ -64,7 +70,7 @@ class ApiComposer:
     def obtain_model(self, **common_dict):
         self.best_models = None
         self.history = None
-
+        preset = common_dict['preset']
         # Prepare parameters
         api_params_dict, composer_params_dict, tuner_params_dict = _divide_parameters(common_dict)
 
@@ -72,19 +78,19 @@ class ApiComposer:
         self.current_model, self.best_models, self.history = self.compose_fedot_model(
             api_params=api_params_dict,
             composer_params=composer_params_dict,
-            tuning_params=tuner_params_dict)
+            tuning_params=tuner_params_dict,
+            preset=preset)
 
         if isinstance(self.best_models, tools.ParetoFront):
             self.best_models.__class__ = ParetoFront
             self.best_models.objective_names = common_dict['composer_metric']
 
-        # Final fit for obtained pipeline on full dataset
-        self.current_model.fit_from_scratch(common_dict['train_data'])
-
         return self.current_model, self.best_models, self.history
 
     def get_gp_composer_builder(self, task: Task,
                                 metric_function,
+                                preset: str,
+                                full_minutes_timeout: int,
                                 composer_requirements: PipelineComposerRequirements,
                                 optimiser: Type[GraphOptimiser],
                                 optimizer_parameters: GraphOptimiserParameters,
@@ -98,6 +104,8 @@ class ApiComposer:
 
         :param task: task for solving
         :param metric_function: function for individuals evaluating
+        :param preset: name of using preset
+        :param full_minutes_timeout: number of minutes for all AutoML
         :param composer_requirements: params for composer
         :param optimiser: optimiser for composer
         :param optimizer_parameters: params for optimizer
@@ -113,7 +121,7 @@ class ApiComposer:
             with_metrics(metric_function).with_logger(logger)
 
         if initial_assumption is None:
-            initial_pipelines = self.initial_assumptions.get_initial_assumption(data, task)
+            initial_pipelines = AssumptionsBuilder.get(task, data).build()
         elif isinstance(initial_assumption, Pipeline):
             initial_pipelines = [initial_assumption]
         else:
@@ -122,9 +130,15 @@ class ApiComposer:
                 raise ValueError(f'{prefix}: List[Pipeline] or Pipeline needed, but has {type(initial_assumption)}')
             initial_pipelines = initial_assumption
         # Check initial assumption
-        fit_and_check_correctness(initial_pipelines, data, logger=logger)
+        fitted_pipeline, fit_time = fit_and_check_correctness(initial_pipelines[0], data, logger=logger,
+                                                              n_jobs=composer_requirements.n_jobs)
         builder = builder.with_initial_pipelines(initial_pipelines)
-        return builder
+
+        # Update builder and preset if required
+        builder, new_preset = update_builder(builder, composer_requirements, fit_time, full_minutes_timeout, preset)
+        # Store information about preset
+        self.preset_name = new_preset
+        return builder, fitted_pipeline, fit_time
 
     def divide_operations(self,
                           available_operations,
@@ -153,15 +167,45 @@ class ApiComposer:
             secondary_operations = available_operations
         return primary_operations, secondary_operations
 
-    def compose_fedot_model(self, api_params: dict, composer_params: dict, tuning_params: dict):
+    def _set_initial_assumption(self, api_params: dict, composer_params: dict):
+        """ Generates and sets an initial assumption, if not given, from the given available operations
+        Also, since it is impossible to form a valid pipeline for the time series problem without 'lagged' operation,
+        if it is not in the list of available ones, it is added """
+
+        if api_params['task'] == TaskTypesEnum.ts_forecasting and \
+                'lagged' not in api_params['available_operations']:
+            api_params['available_operations'].append('lagged')
+
+        if not api_params['initial_assumption']:
+            is_input = isinstance(api_params['train_data'], InputData)
+            if is_input and api_params['train_data'].data_type == DataTypesEnum.multi_ts:
+                # It is needed to use data operations also for multi_ts data
+                repository_name = 'all'
+            else:
+                repository_name = 'model'
+            assumptions_builder = AssumptionsBuilder(task=api_params['task'],
+                                                     data=api_params['train_data'],
+                                                     repository_name=repository_name)\
+                .get(api_params['task'], api_params['train_data'])\
+                .with_logger(api_params['logger'])\
+                .from_operations(composer_params['available_operations'])
+            api_params['initial_assumption'] = assumptions_builder.build()
+
+    def compose_fedot_model(self, api_params: dict, composer_params: dict, tuning_params: dict,
+                            preset: str):
         """ Function for composing FEDOT pipeline model """
+        # Initialize timer for all AutoMl operations
+        self.timer = ApiTime(time_for_automl=api_params['timeout'],
+                             with_tuning=tuning_params['with_tuning'])
 
         metric_function = self.obtain_metric(api_params['task'], composer_params['composer_metric'])
 
-        if composer_params['available_operations'] is None:
+        if not composer_params['available_operations']:
             composer_params['available_operations'] = get_operations_for_task(api_params['task'], mode='model')
+        else:
+            self._set_initial_assumption(api_params=api_params, composer_params=composer_params)
 
-        api_params['logger'].message('Composition started. Parameters tuning: {}. ''Set of candidate models: {}. '
+        api_params['logger'].message('AutoML started. Parameters tuning: {}. ''Set of candidate models: {}. '
                                      'Time limit: {} min'.format(tuning_params['with_tuning'],
                                                                  composer_params['available_operations'],
                                                                  api_params['timeout']))
@@ -169,23 +213,17 @@ class ApiComposer:
         primary_operations, secondary_operations = self.divide_operations(composer_params['available_operations'],
                                                                           api_params['task'])
 
-        if api_params['timeout'] is None:
-            timeout_for_composing = None
-        else:
-            timeout_for_composing = api_params['timeout'] / 2 if tuning_params['with_tuning'] else api_params['timeout']
-            timeout_for_composing = datetime.timedelta(minutes=timeout_for_composing)
-        starting_time_for_composing = datetime.datetime.now()
         # the choice and initialisation of the GP composer
-        composer_requirements = \
-            PipelineComposerRequirements(primary=primary_operations,
-                                         secondary=secondary_operations,
-                                         max_arity=composer_params['max_arity'],
-                                         max_depth=composer_params['max_depth'],
-                                         pop_size=composer_params['pop_size'],
-                                         num_of_generations=composer_params['num_of_generations'],
-                                         cv_folds=composer_params['cv_folds'],
-                                         validation_blocks=composer_params['validation_blocks'],
-                                         timeout=timeout_for_composing)
+        composer_requirements = PipelineComposerRequirements(primary=primary_operations,
+                                                             secondary=secondary_operations,
+                                                             max_arity=composer_params['max_arity'],
+                                                             max_depth=composer_params['max_depth'],
+                                                             pop_size=composer_params['pop_size'],
+                                                             num_of_generations=composer_params['num_of_generations'],
+                                                             cv_folds=composer_params['cv_folds'],
+                                                             validation_blocks=composer_params['validation_blocks'],
+                                                             timeout=self.timer.datetime_composing,
+                                                             n_jobs=api_params['n_jobs'])
 
         genetic_scheme_type = GeneticSchemeTypesEnum.parameter_free
 
@@ -213,20 +251,33 @@ class ApiComposer:
         if 'optimizer_external_params' in composer_params:
             self.optimizer_external_parameters = composer_params['optimizer_external_params']
 
-        builder = self.get_gp_composer_builder(task=api_params['task'],
-                                               metric_function=metric_function,
-                                               composer_requirements=composer_requirements,
-                                               optimiser=self.optimiser,
-                                               optimizer_parameters=optimizer_parameters,
-                                               optimizer_external_parameters=self.optimizer_external_parameters,
-                                               data=api_params['train_data'],
-                                               initial_assumption=api_params['initial_assumption'],
-                                               logger=api_params['logger'])
-
-        gp_composer = builder.build()
-
-        api_params['logger'].message('Pipeline composition started')
-        pipeline_gp_composed = gp_composer.compose_pipeline(data=api_params['train_data'])
+        builder, fitted_initial_pipeline, init_pipeline_fit_time = \
+            self.get_gp_composer_builder(task=api_params['task'],
+                                         metric_function=metric_function,
+                                         preset=preset,
+                                         full_minutes_timeout=api_params['timeout'],
+                                         composer_requirements=composer_requirements,
+                                         optimiser=self.optimiser,
+                                         optimizer_parameters=optimizer_parameters,
+                                         optimizer_external_parameters=self.optimizer_external_parameters,
+                                         data=api_params['train_data'],
+                                         initial_assumption=api_params['initial_assumption'],
+                                         logger=api_params['logger'])
+        gp_composer = None
+        timeout_were_set = self.timer.datetime_composing is not None
+        if timeout_were_set and init_pipeline_fit_time >= self.timer.datetime_composing / composer_params['pop_size']:
+            api_params['logger'].message(f'Timeout is too small for composing '
+                                         f'because fit_time is {init_pipeline_fit_time.total_seconds()} sec.,'
+                                         ' so it is skipped.')
+            # Use initial pipeline as final solution
+            pipeline_gp_composed = fitted_initial_pipeline
+        else:
+            # Launch pipeline structure composition
+            with self.timer.launch_composing():
+                gp_composer = builder.build()
+                api_params['logger'].message(f'Pipeline composition started. Initial pipeline was fitted '
+                                             f'for fit_time {init_pipeline_fit_time.total_seconds()} sec.')
+                pipeline_gp_composed = gp_composer.compose_pipeline(data=api_params['train_data'])
 
         if isinstance(pipeline_gp_composed, list):
             for pipeline in pipeline_gp_composed:
@@ -237,50 +288,21 @@ class ApiComposer:
             best_candidates = [pipeline_gp_composed]
             pipeline_gp_composed.log = api_params['logger']
 
-        spending_time_for_composing = datetime.datetime.now() - starting_time_for_composing
+        pipeline_gp_composed = self.tune_final_pipeline(api_params=api_params, tuning_params=tuning_params,
+                                                        composer_requirements=composer_requirements,
+                                                        pipeline_gp_composed=pipeline_gp_composed,
+                                                        timer=self.timer,
+                                                        init_pipeline_fit_time=init_pipeline_fit_time)
 
-        if tuning_params['with_tuning']:
-            if tuning_params['tuner_metric'] is None:
-                # Default metric for tuner
-                tune_metrics = TunerMetricByTask(api_params['task'].task_type)
-                tuner_loss, loss_params = tune_metrics.get_metric_and_params(api_params['train_data'])
-                api_params['logger'].message(f'Tuner metric is None, '
-                                             f'{tuner_loss.__name__} was set as default')
-            else:
-                # Get metric and parameters by name
-                tuner_loss, loss_params = self.tuner_metric_by_name(metric_name=tuning_params['tuner_metric'],
-                                                                    train_data=api_params['train_data'],
-                                                                    task=api_params['task'])
+        api_params['logger'].message('Model generation finished')
 
-            iterations = 20 if api_params['timeout'] is None else 1000
-            timeout_in_sec = datetime.timedelta(minutes=api_params['timeout']).total_seconds()
-            timeout_for_tuning = timeout_in_sec - spending_time_for_composing.total_seconds()
-
-            if timeout_for_tuning < 15:
-                api_params['logger'].info(f'Time for pipeline composing  was {str(spending_time_for_composing)}.'
-                                          f'The remaining {timeout_for_tuning} seconds are not enough '
-                                          f'to tune the hyperparameters.'
-                                          f'Composed pipeline will be returned without tuning hyperparameters.')
-            else:
-                # Tune all nodes in the pipeline
-                api_params['logger'].message('Hyperparameters tuning started')
-                vb_number = composer_requirements.validation_blocks
-                folds = composer_requirements.cv_folds
-                pipeline_gp_composed = pipeline_gp_composed.fine_tune_all_nodes(loss_function=tuner_loss,
-                                                                                loss_params=loss_params,
-                                                                                input_data=api_params['train_data'],
-                                                                                iterations=iterations,
-                                                                                timeout=round(timeout_for_tuning / 60),
-                                                                                cv_folds=folds,
-                                                                                validation_blocks=vb_number)
-
-        api_params['logger'].message('Model composition finished')
-
-        history = gp_composer.optimiser.history
+        if gp_composer is not None:
+            history = gp_composer.optimiser.history
+        else:
+            history = None
 
         # enforce memory cleaning
         gc.collect()
-
         return pipeline_gp_composed, best_candidates, history
 
     def tuner_metric_by_name(self, metric_name, train_data: InputData, task: Task):
@@ -315,18 +337,67 @@ class ApiComposer:
 
         return loss_function, loss_params
 
+    def tune_final_pipeline(self, api_params, tuning_params,
+                            composer_requirements, pipeline_gp_composed,
+                            timer: ApiTime, init_pipeline_fit_time):
+        """ Launch tuning procedure for obtained pipeline by composer """
+        if tuning_params['with_tuning'] is False:
+            # Return source pipeline
+            return pipeline_gp_composed
 
-def fit_and_check_correctness(initial_pipelines: List[Pipeline],
+        timeout_for_tuning = timer.determine_resources_for_tuning(init_pipeline_fit_time)
+
+        if timeout_for_tuning < MINIMAL_SECONDS_FOR_TUNING:
+            api_params['logger'].info(f'Time for pipeline composing was {str(timer.composing_spend_time)}.\n'
+                                      f'The remaining {max(0, timeout_for_tuning)} seconds are not enough '
+                                      f'to tune the hyperparameters.')
+            api_params['logger'].info('Composed pipeline returned without tuning.')
+        else:
+            if tuning_params['tuner_metric'] is None:
+                # Default metric for tuner
+                tune_metrics = TunerMetricByTask(api_params['task'].task_type)
+                tuner_loss, loss_params = tune_metrics.get_metric_and_params(api_params['train_data'])
+                api_params['logger'].message(f'Tuner metric is None, '
+                                             f'{tuner_loss.__name__} was set as default')
+            else:
+                # Get metric and parameters by name
+                tuner_loss, loss_params = self.tuner_metric_by_name(metric_name=tuning_params['tuner_metric'],
+                                                                    train_data=api_params['train_data'],
+                                                                    task=api_params['task'])
+
+            # Tune all nodes in the pipeline
+            with timer.launch_tuning():
+                api_params['logger'].message('Hyperparameters tuning started')
+                vb_number = composer_requirements.validation_blocks
+                folds = composer_requirements.cv_folds
+                timeout_for_tuning = abs(timeout_for_tuning) / 60
+                pipeline_gp_composed = pipeline_gp_composed.\
+                    fine_tune_all_nodes(loss_function=tuner_loss,
+                                        loss_params=loss_params,
+                                        input_data=api_params['train_data'],
+                                        iterations=DEFAULT_TUNING_ITERATIONS_NUMBER,
+                                        timeout=timeout_for_tuning,
+                                        cv_folds=folds,
+                                        validation_blocks=vb_number)
+                api_params['logger'].message('Hyperparameters tuning finished')
+        return pipeline_gp_composed
+
+
+def fit_and_check_correctness(pipeline: Pipeline,
                               data: Union[InputData, MultiModalData],
-                              logger: Log):
+                              logger: Log, n_jobs=1):
     """ Test is initial pipeline can be fitted on presented data and give predictions """
     try:
         _, data_test = train_test_data_setup(data)
-        initial_pipelines[0].fit(data)
-        initial_pipelines[0].predict(data_test)
+        start_init_fit = datetime.datetime.now()
 
-        message_success = 'Initial pipeline was fitted successfully'
-        logger.debug(message_success)
+        logger.message('Initial pipeline fitting started')
+
+        pipeline.fit(data, n_jobs=n_jobs)
+        pipeline.predict(data_test)
+
+        fit_time = datetime.datetime.now() - start_init_fit
+        logger.message('Initial pipeline was fitted successfully')
     except Exception as ex:
         fit_failed_info = f'Initial pipeline fit was failed due to: {ex}.'
         advice_info = f'{fit_failed_info} Check pipeline structure and the correctness of the data'
@@ -334,6 +405,7 @@ def fit_and_check_correctness(initial_pipelines: List[Pipeline],
         logger.info(fit_failed_info)
         print(traceback.format_exc())
         raise ValueError(advice_info)
+    return pipeline, fit_time
 
 
 def _divide_parameters(common_dict: dict) -> List[dict]:
@@ -341,7 +413,7 @@ def _divide_parameters(common_dict: dict) -> List[dict]:
 
     :param common_dict: dictionary with parameters for all AutoML modules
     """
-    api_params_dict = dict(train_data=None, task=Task, logger=Log, timeout=5, initial_assumption=None)
+    api_params_dict = dict(train_data=None, task=Task, logger=Log, timeout=5, initial_assumption=None, n_jobs=1)
 
     composer_params_dict = dict(max_depth=None, max_arity=None, pop_size=None, num_of_generations=None,
                                 available_operations=None, composer_metric=None, validation_blocks=None,
