@@ -3,6 +3,8 @@ from typing import Optional, Dict, Union
 
 import numpy as np
 
+from fedot.core.constants import DEFAULT_FORECAST_LENGTH, \
+    DEFAULT_API_TIMEOUT_MINUTES, AUTO_PRESET_NAME
 from fedot.api.api_utils.presets import OperationsPreset
 from fedot.core.data.data import InputData
 from fedot.core.data.multi_modal import MultiModalData
@@ -13,7 +15,6 @@ from fedot.core.repository.tasks import Task, TaskTypesEnum, TsForecastingParams
 class ApiParams:
 
     def __init__(self):
-        self.default_forecast_length = 30
         self.api_params = None
         self.log = None
         self.task = None
@@ -22,35 +23,26 @@ class ApiParams:
         self.metric_name = None
         self.initial_assumption = None
 
-    def check_input_params(self, **input_params):
-        self.metric_to_compose = None
-        self.api_params['problem'] = input_params['problem']
-        self.log = default_log('FEDOT logger', verbose_level=input_params['verbose_level'])
+    def initialize_params(self, **input_params):
+        """ Merge input_params dictionary with several parameters for AutoML algorithm """
+        self.get_initial_params(**input_params)
+        preset_operations = OperationsPreset(task=self.task, preset_name=input_params['preset'])
+        self.api_params = preset_operations.composer_params_based_on_preset(composer_params=self.api_params)
 
-        if input_params['seed'] is not None:
-            np.random.seed(input_params['seed'])
-            random.seed(input_params['seed'])
-
-        if 'metric' in self.api_params:
-            self.api_params['composer_metric'] = self.api_params['metric']
-            del self.api_params['metric']
-            self.metric_to_compose = self.api_params['composer_metric']
-
-        if input_params['problem'] == 'ts_forecasting' and input_params['task_params'] is None:
-            self.log.warn('The value of the forecast depth was set to {}.'.format(self.default_forecast_length))
-            self.task_params = TsForecastingParams(forecast_length=self.default_forecast_length)
-
-        if input_params['problem'] == 'clustering':
-            raise ValueError('This type of task is not not supported in API now')
+        # Final check for correctness for timeout and generations
+        self.api_params = check_timeout_vs_generations(self.api_params)
 
     def get_initial_params(self, **input_params):
+        merge_and_correct_timeout_params(input_params)
         if input_params['composer_params'] is None:
             self.api_params = self.get_default_evo_params(problem=input_params['problem'])
         else:
+            if input_params.get('preset') is not None:
+                input_params['composer_params']['preset'] = input_params['preset']
             self.api_params = {**self.get_default_evo_params(problem=input_params['problem']),
                                **input_params['composer_params']}
 
-        self.check_input_params(**input_params)
+        self._check_input_params(**input_params)
 
         self.task = self.get_task_params(input_params['problem'],
                                          input_params['task_params'])
@@ -60,15 +52,12 @@ class ApiParams:
             'task': self.task,
             'logger': self.log,
             'metric_name': self.metric_name,
-            'composer_metric': self.metric_to_compose
+            'composer_metric': self.metric_to_compose,
+            'timeout': input_params['timeout'],
+            'current_model': None
         }
         self.api_params = {**self.api_params, **param_dict}
 
-    def initialize_params(self, **input_params):
-        self.get_initial_params(**input_params)
-        preset_operations = OperationsPreset(task=self.task, preset_name=input_params['preset'])
-        self.api_params = preset_operations.composer_params_based_on_preset(composer_params=self.api_params)
-
     def accept_and_apply_recommendations(self, input_data: Union[InputData, MultiModalData], recommendations: Dict):
         """
         Accepts recommendations for api params from DataAnalyser
@@ -94,7 +83,9 @@ class ApiParams:
         else:
             preset_name = '*tree'
         preset_operations = OperationsPreset(task=task, preset_name=preset_name)
-        del self.api_params['available_operations']
+
+        if self.api_params.get('available_operations') is not None:
+            del self.api_params['available_operations']
         self.api_params = preset_operations.composer_params_based_on_preset(composer_params=self.api_params)
         param_dict = {
             'task': self.task,
@@ -104,16 +95,38 @@ class ApiParams:
         }
         self.api_params = {**self.api_params, **param_dict}
 
+    def _check_input_params(self, **input_params):
+        """ Filter different parameters and search for conflicts """
+        self.metric_to_compose = None
+        self.api_params['problem'] = input_params['problem']
+        self.log = default_log('FEDOT logger', verbose_level=input_params['verbose_level'])
+
+        if input_params['seed'] is not None:
+            np.random.seed(input_params['seed'])
+            random.seed(input_params['seed'])
+
+        if 'metric' in self.api_params:
+            self.api_params['composer_metric'] = self.api_params['metric']
+            del self.api_params['metric']
+            self.metric_to_compose = self.api_params['composer_metric']
+
+        if input_params['problem'] == 'ts_forecasting' and input_params['task_params'] is None:
+            self.log.warn('The value of the forecast depth was set to {}.'.format(DEFAULT_FORECAST_LENGTH))
+            self.task_params = TsForecastingParams(forecast_length=DEFAULT_FORECAST_LENGTH)
+
+        if input_params['problem'] == 'clustering':
+            raise ValueError('This type of task is not not supported in API now')
+
     @staticmethod
     def get_default_evo_params(problem: str):
         """ Dictionary with default parameters for composer """
-        params = {'max_depth': 3,
-                  'max_arity': 4,
+        params = {'max_depth': 6,
+                  'max_arity': 3,
                   'pop_size': 20,
-                  'num_of_generations': 20,
+                  'num_of_generations': 100,
                   'timeout': 2,
                   'with_tuning': True,
-                  'preset': 'best_quality',
+                  'preset': AUTO_PRESET_NAME,
                   'genetic_scheme': None,
                   'history_folder': None,
                   'stopping_after_n_generation': 10}
@@ -145,3 +158,31 @@ class ApiParams:
                      'ts_forecasting': Task(TaskTypesEnum.ts_forecasting, task_params=task_params)
                      }
         return task_dict[problem]
+
+
+def check_timeout_vs_generations(api_params):
+    timeout = api_params['timeout']
+    num_of_generations = api_params['num_of_generations']
+    if timeout in [-1, None]:
+        api_params['timeout'] = None
+        if num_of_generations is None:
+            raise ValueError('"num_of_generations" should be specified if infinite "timeout" is given')
+    elif timeout > 0:
+        if num_of_generations is None:
+            api_params['num_of_generations'] = 10000
+    else:
+        raise ValueError(f'invalid "timeout" value: timeout={timeout}')
+    return api_params
+
+
+def merge_and_correct_timeout_params(input_params):
+    """ Merge timeout parameter from composer parameters and from API """
+    if input_params['composer_params'] is None:
+        # Composer parameters were skipped during initialisation
+        return input_params
+
+    composer_parameters = input_params['composer_params']
+    if composer_parameters.get('timeout') is not None:
+        if np.isclose(input_params['timeout'], DEFAULT_API_TIMEOUT_MINUTES):
+            # Replace default API values with composer dictionary value
+            input_params['timeout'] = composer_parameters['timeout']
