@@ -4,18 +4,20 @@ from typing import List, Optional, Tuple, Union
 import numpy as np
 import pandas as pd
 
+from fedot.api.api_utils.api_composer import ApiComposer, fit_and_check_correctness
+from fedot.api.api_utils.api_data import ApiDataProcessor
 from fedot.api.api_utils.api_data_analyser import DataAnalyser
+from fedot.api.api_utils.assumptions.assumptions_builder import AssumptionsBuilder
+from fedot.api.api_utils.metrics import ApiMetrics
+from fedot.api.api_utils.params import ApiParams
+from fedot.core.constants import DEFAULT_API_TIMEOUT_MINUTES
 from fedot.core.data.data import InputData, OutputData
 from fedot.core.data.multi_modal import MultiModalData
-from fedot.core.data.visualisation import plot_biplot, plot_roc_auc, plot_forecast
+from fedot.core.data.visualisation import plot_biplot, plot_forecast, plot_roc_auc
 from fedot.core.pipelines.node import PrimaryNode
 from fedot.core.pipelines.pipeline import Pipeline
 from fedot.core.repository.quality_metrics_repository import MetricsRepository
 from fedot.core.repository.tasks import TaskParams, TaskTypesEnum
-from fedot.api.api_utils.params import ApiParams
-from fedot.api.api_utils.api_data import ApiDataProcessor
-from fedot.api.api_utils.metrics import ApiMetrics
-from fedot.api.api_utils.api_composer import ApiComposer, fit_and_check_correctness
 from fedot.explainability.explainers import explain_pipeline
 from fedot.preprocessing.preprocessing import merge_preprocessors
 from fedot.remote.remote_evaluator import RemoteEvaluator
@@ -27,7 +29,6 @@ class Fedot:
     """
     Main class for FEDOT API.
     Facade for ApiDataProcessor, ApiComposer, ApiMetrics, ApiInitialAssumptions.
-
     :param problem: the name of modelling problem to solve:
         - classification
         - regression
@@ -39,6 +40,7 @@ class Fedot:
             (data operations) that only reduce the dimensionality of the data, but cannot increase
              it. For example, there are no polynomial features and one-hot encoding operations
         - 'stable' - The most reliable preset in which the most stable operations are included.
+        - 'auto' - Automatically determine which preset should be used.
         - 'gpu' - Models that use GPU resources for computation.
         - 'ts' - A special preset with models for time series forecasting task.
         - 'automl' - A special preset with only AutoML libraries such as TPOT and H2O as operations.
@@ -70,17 +72,19 @@ class Fedot:
     :param safe_mode: if set True it will cut large datasets to prevent memory overflow and use label encoder
     instead of oneHot encoder if summary cardinality of categorical features is high.
     :param initial_assumption: initial assumption for composer
+    :param n_jobs: num of n_jobs for parallelization (-1 for use all cpu's)
     """
 
     def __init__(self,
                  problem: str,
                  preset: str = None,
-                 timeout: Optional[float] = 5.0,
+                 timeout: Optional[float] = DEFAULT_API_TIMEOUT_MINUTES,
                  composer_params: dict = None,
                  task_params: TaskParams = None,
                  seed=None, verbose_level: int = 0,
                  safe_mode=True,
-                 initial_assumption: Union[Pipeline, List[Pipeline]] = None
+                 initial_assumption: Union[Pipeline, List[Pipeline]] = None,
+                 n_jobs: int = 1
                  ):
 
         # Classes for dealing with metrics, data sources and hyperparameters
@@ -88,21 +92,20 @@ class Fedot:
         self.api_composer = ApiComposer(problem)
         self.params = ApiParams()
 
-        input_params = {'problem': problem, 'preset': preset,
+        # Define parameters, that were set via init in init
+        input_params = {'problem': problem, 'preset': preset, 'timeout': timeout,
                         'composer_params': composer_params, 'task_params': task_params,
-                        'seed': seed, 'verbose_level': verbose_level}
+                        'seed': seed, 'verbose_level': verbose_level,
+                        'initial_assumption': initial_assumption}
         self.params.initialize_params(**input_params)
-        self.params.api_params['current_model'] = None
 
+        # Get metrics for optimization
         metric_name = self.params.api_params['metric_name']
         self.task_metrics, self.composer_metrics, self.tuner_metrics = self.metrics.get_metrics_for_task(metric_name)
         self.params.api_params['tuner_metric'] = self.tuner_metrics
+        self.params.api_params['n_jobs'] = n_jobs
 
-        # Update timeout, num_of_generations and initial_assumption parameters
-        if composer_params is not None and 'timeout' in composer_params:
-            timeout = composer_params['timeout']
-        self.update_params(timeout, self.params.api_params['num_of_generations'], initial_assumption)
-        self.timeout_set_in_init = self.params.api_params['timeout']
+        # Initialize data processors for data preprocessing and preliminary data analysis
         self.data_processor = ApiDataProcessor(task=self.params.api_params['task'],
                                                log=self.params.api_params['logger'])
         self.data_analyser = DataAnalyser(safe_mode=safe_mode)
@@ -121,7 +124,6 @@ class Fedot:
             predefined_model: Union[str, Pipeline] = None):
         """
         Fit the graph with a predefined structure or compose and fit the new graph
-
         :param features: the array with features of train data
         :param target: the array with target values of train data
         :param predefined_model: the name of the atomic model or Pipeline instance.
@@ -144,16 +146,22 @@ class Fedot:
             # Fit predefined model and return it without composing
             self.current_pipeline = self._process_predefined_model(predefined_model)
         else:
-            if self.timeout_set_in_init is not None:
-                self.params.api_params['timeout'] = self.timeout_set_in_init
             self.current_pipeline, self.best_models, self.history = self.api_composer.obtain_model(
                 **self.params.api_params)
 
-        self._train_pipeline_on_full_dataset(recommendations, full_train_not_preprocessed)
+        # Final fit for obtained pipeline on full dataset
+        if self.history is not None:
+            self._train_pipeline_on_full_dataset(recommendations, full_train_not_preprocessed)
+            self.params.api_params['logger'].message('Final pipeline was fitted')
+        else:
+            self.params.api_params['logger'].message('Already fitted initial pipeline is used')
 
         # Store data encoder in the pipeline if it is required
         self.current_pipeline.preprocessor = merge_preprocessors(self.data_processor.preprocessor,
                                                                  self.current_pipeline.preprocessor)
+
+        self.params.api_params['logger'].message(f'Final pipeline: {str(self.current_pipeline)}')
+
         return self.current_pipeline
 
     def predict(self,
@@ -161,7 +169,6 @@ class Fedot:
                 save_predictions: bool = False):
         """
         Predict new target using already fitted model
-
         :param features: the array with features of test data
         :param save_predictions: if True-save predictions as csv-file in working directory.
         :return: the array with prediction values
@@ -185,7 +192,6 @@ class Fedot:
                       probs_for_all_classes: bool = False):
         """
         Predict the probability of new target using already fitted classification model
-
         :param features: the array with features of test data
         :param save_predictions: if True-save predictions as csv-file in working directory.
         :param probs_for_all_classes: return probability for each class even for binary case
@@ -216,7 +222,6 @@ class Fedot:
                  save_predictions: bool = False):
         """
         Forecast the new values of time series
-
         :param pre_history: the array with features for pre-history of the forecast
         :param forecast_length: num of steps to forecast
         :param save_predictions: if True-save predictions as csv-file in working directory.
@@ -248,7 +253,6 @@ class Fedot:
     def load(self, path):
         """
         Load saved graph from disk
-
         :param path to json file with model
         """
         self.current_pipeline.load(path)
@@ -278,7 +282,6 @@ class Fedot:
                     metric_names: Union[str, List[str]] = None) -> dict:
         """
         Get quality metrics for the fitted graph
-
         :param target: the array with target values of test data
         :param metric_names: the names of required metrics
         :return: the values of quality metrics
@@ -319,7 +322,6 @@ class Fedot:
 
                 metric_value = abs(metric_cls.metric(reference=real,
                                                      predicted=prediction))
-
                 calculated_metrics[metric_name] = metric_value
 
         return calculated_metrics
@@ -335,48 +337,9 @@ class Fedot:
         self.params.api_params['logger'].info('Predictions was saved in current directory.')
 
     def update_params(self, timeout, num_of_generations, initial_assumption):
-        if timeout in [-1, None]:
-            self.params.api_params['timeout'] = None
-            if num_of_generations is None:
-                raise ValueError('"num_of_generations" should be specified if infinite "timeout" is given')
-            self.params.api_params['num_of_generations'] = num_of_generations
-        elif timeout > 0:
-            self.params.api_params['timeout'] = timeout
-            if num_of_generations is not None:
-                self.params.api_params['num_of_generations'] = num_of_generations
-            else:
-                self.params.api_params['num_of_generations'] = 10000
-        else:
-            raise ValueError(f'invalid "timeout" value: timeout={timeout}')
-
         if initial_assumption is not None:
             self.params.api_params['initial_assumption'] = initial_assumption
 
-    def _init_remote_if_necessary(self):
-        remote = RemoteEvaluator()
-        if remote.use_remote and remote.remote_task_params is not None:
-            task = self.params.api_params['task']
-            if task.task_type == TaskTypesEnum.ts_forecasting:
-                task_str = \
-                    f'Task(TaskTypesEnum.ts_forecasting, ' \
-                    f'TsForecastingParams(forecast_length={task.task_params.forecast_length}))'
-            else:
-                task_str = f'Task({str(task.task_type)})'
-            remote.remote_task_params.task_type = task_str
-            remote.remote_task_params.is_multi_modal = isinstance(self.train_data, MultiModalData)
-
-            if isinstance(self.target, str):
-                remote.remote_task_params.target = self.target
-
-    def _train_pipeline_on_full_dataset(self, recommendations: dict, full_train_not_preprocessed):
-        """ Apply training procedure for obtained pipeline if dataset was clipped """
-        if 'cut' in recommendations:
-            # if data was cut we need to refit pipeline on full data
-            self.data_processor.accept_and_apply_recommendations(full_train_not_preprocessed,
-                                                                 {k: v for k, v in recommendations.items()
-                                                                  if k != 'cut'})
-            self.current_pipeline.fit(full_train_not_preprocessed)
-
     def explain(self, features: Union[str, np.ndarray, pd.DataFrame, InputData, dict] = None,
                 method: str = 'surrogate_dt', visualize: bool = True, **kwargs) -> 'Explainer':
         """Create explanation for 'current_pipeline' according to the selected 'method'.
@@ -397,6 +360,31 @@ class Fedot:
 
         return explainer
 
+    def _init_remote_if_necessary(self):
+        remote = RemoteEvaluator()
+        if remote.use_remote and remote.remote_task_params is not None:
+            task = self.params.api_params['task']
+            if task.task_type == TaskTypesEnum.ts_forecasting:
+                task_str = \
+                    f'Task(TaskTypesEnum.ts_forecasting, ' \
+                    f'TsForecastingParams(forecast_length={task.task_params.forecast_length}))'
+            else:
+                task_str = f'Task({str(task.task_type)})'
+            remote.remote_task_params.task_type = task_str
+            remote.remote_task_params.is_multi_modal = isinstance(self.train_data, MultiModalData)
+
+            if isinstance(self.target, str):
+                remote.remote_task_params.target = self.target
+
+    def _train_pipeline_on_full_dataset(self, recommendations: dict, full_train_not_preprocessed):
+        """ Apply training procedure for obtained pipeline if dataset was clipped """
+        if recommendations:
+            # if data was cut we need to refit pipeline on full data
+            self.data_processor.accept_and_apply_recommendations(full_train_not_preprocessed,
+                                                                 {k: v for k, v in recommendations.items()
+                                                                  if k != 'cut'})
+        self.current_pipeline.fit(full_train_not_preprocessed, n_jobs=self.params.api_params['n_jobs'])
+
     def _process_predefined_model(self, predefined_model):
         """ Fit and return predefined model """
 
@@ -404,15 +392,16 @@ class Fedot:
             pipelines = [predefined_model]
         elif predefined_model == 'auto':
             # Generate initial assumption automatically
-            pipelines = self.api_composer.initial_assumptions.get_initial_assumption(self.train_data,
-                                                                                     self.params.api_params['task'])
+            pipelines = AssumptionsBuilder.get(self.params.api_params['task'], self.train_data).build()
         elif isinstance(predefined_model, str):
             model = PrimaryNode(predefined_model)
             pipelines = [Pipeline(model)]
         else:
             raise ValueError(f'{type(predefined_model)} is not supported as Fedot model')
 
+        final_pipeline = pipelines[0]
         # Perform fitting
-        fit_and_check_correctness(pipelines, data=self.train_data,
-                                  logger=self.params.api_params['logger'])
-        return pipelines[0]
+        final_pipeline, _ = fit_and_check_correctness(final_pipeline, data=self.train_data,
+                                                      logger=self.params.api_params['logger'],
+                                                      n_jobs=self.params.api_params['n_jobs'])
+        return final_pipeline
