@@ -1,7 +1,7 @@
 from typing import List, Optional
 
 from fedot.core.repository.operation_types_repository import get_operations_for_task
-from fedot.core.utils import ComparableEnum as Enum
+from fedot.core.utilities.data_structures import ComparableEnum as Enum
 
 
 class RemoveType(Enum):
