@@ -1,12 +1,12 @@
 import math
 from copy import deepcopy
-from dataclasses import dataclass
 from functools import partial
+from itertools import zip_longest
 from typing import (Any, Callable, List, Optional, Tuple, Union)
 
 import numpy as np
-from tqdm import tqdm
 from deap.tools import ParetoFront
+from tqdm import tqdm
 
 from fedot.core.composer.constraint import constraint_function
 from fedot.core.log import Log
@@ -127,11 +127,12 @@ class EvoGraphOptimiser(GraphOptimiser):
 
         self.population = None
         self.initial_graph = initial_graph
+        self.prev_best = None
 
-    def _create_randomized_pop_from_inital_graph(self, initial_graphs: List[OptGraph]) -> List[Individual]:
+    def _create_randomized_pop(self, individuals: List[Individual]) -> List[Individual]:
         """
         Fill first population with mutated variants of the initial_graphs
-        :param initial_graphs: Initial assumption for first population
+        :param individuals: Initial assumption for first population
         :return: list of individuals
         """
         initial_req = deepcopy(self.requirements)
@@ -139,31 +140,36 @@ class EvoGraphOptimiser(GraphOptimiser):
         randomized_pop = []
         n_iter = self.requirements.pop_size * 10
         while n_iter > 0:
-            initial_graph = np.random.choice(initial_graphs)
+            initial_individual = np.random.choice(individuals)
             n_iter -= 1
             new_ind = mutation(types=self.parameters.mutation_types,
                                params=self.graph_generation_params,
-                               ind=Individual(deepcopy(initial_graph)),
+                               ind=initial_individual,
                                requirements=initial_req,
-                               max_depth=self.max_depth, log=self.log,
-                               add_to_history=False)
+                               max_depth=self.max_depth, log=self.log)
             if new_ind not in randomized_pop:
                 # to suppress duplicated
                 randomized_pop.append(new_ind)
 
-            if len(randomized_pop) == self.requirements.pop_size - len(initial_graphs):
+            if len(randomized_pop) == self.requirements.pop_size - len(individuals):
                 break
 
         # add initial graph to population
-        for initial_graph in initial_graphs:
-            randomized_pop.append(Individual(deepcopy(initial_graph)))
+        for initial in individuals:
+            randomized_pop.append(initial)
 
         return randomized_pop
 
-    def _init_population(self):
+    def _init_population(self, objective_function, timer):
         if self.initial_graph:
-            adapted_graphs = [self.graph_generation_params.adapter.adapt(g) for g in self.initial_graph]
-            self.population = self._create_randomized_pop_from_inital_graph(adapted_graphs)
+            initial_individuals = [Individual(self.graph_generation_params.adapter.adapt(g)) for g in
+                                   self.initial_graph]
+            initial_individuals = self._evaluate_individuals(initial_individuals,
+                                                             objective_function=objective_function,
+                                                             timer=timer,
+                                                             n_jobs=self.requirements.n_jobs)
+            self.default_on_next_iteration_callback(initial_individuals)
+            self.population = self._create_randomized_pop(initial_individuals)
         if self.population is None:
             self.population = self._make_population(self.requirements.pop_size)
         return self.population
@@ -174,15 +180,16 @@ class EvoGraphOptimiser(GraphOptimiser):
         if on_next_iteration_callback is None:
             on_next_iteration_callback = self.default_on_next_iteration_callback
 
-        self._init_population()
-
-        num_of_new_individuals = self.offspring_size(offspring_rate)
-
         with OptimisationTimer(log=self.log, timeout=self.requirements.timeout) as t:
+            self._init_population(objective_function, t)
+            num_of_new_individuals = self.offspring_size(offspring_rate)
+
             pbar = tqdm(total=self.requirements.num_of_generations,
                         desc="Generations", unit='gen', initial=1) if show_progress else None
 
-            self.population = self._evaluate_individuals(self.population, objective_function, timer=t)
+            self.population = self._evaluate_individuals(self.population, objective_function,
+                                                         timer=t,
+                                                         n_jobs=self.requirements.n_jobs)
 
             if self.archive is not None:
                 self.archive.update(self.population)
@@ -226,11 +233,12 @@ class EvoGraphOptimiser(GraphOptimiser):
 
                 new_population = []
 
-                for parent_num in range(0, len(selected_individuals), 2):
-                    new_population += self.reproduce(selected_individuals[parent_num],
-                                                     selected_individuals[parent_num + 1])
+                for ind_1, ind_2 in zip_longest(selected_individuals[::2], selected_individuals[1::2]):
+                    new_population += self.reproduce(ind_1, ind_2)
 
-                new_population = self._evaluate_individuals(new_population, objective_function, timer=t)
+                new_population = self._evaluate_individuals(new_population, objective_function,
+                                                            timer=t,
+                                                            n_jobs=self.requirements.n_jobs)
 
                 self.prev_best = deepcopy(self.best_individual)
 
@@ -266,6 +274,9 @@ class EvoGraphOptimiser(GraphOptimiser):
             self.log.info('Result:')
             self.log_info_about_best()
 
+        final_individuals = best if isinstance(best, list) else [best]
+        self.default_on_next_iteration_callback(final_individuals)
+
         output = [ind.graph for ind in best] if isinstance(best, list) else best.graph
 
         return output
@@ -338,8 +349,14 @@ class EvoGraphOptimiser(GraphOptimiser):
                 simpler_equivalents[i] = len(individuals[i].graph.nodes)
         return simpler_equivalents
 
-    def reproduce(self, selected_individual_first, selected_individual_second=None) -> Tuple[Any]:
+    def reproduce(self,
+                  selected_individual_first: Individual,
+                  selected_individual_second: Optional[Individual] = None) -> Tuple[Any]:
+
+        selected_individual_first.parent_operators = []
+
         if selected_individual_second:
+            selected_individual_second.parent_operators = []
             new_inds = crossover(self.parameters.crossover_types,
                                  selected_individual_first,
                                  selected_individual_second,
@@ -355,6 +372,7 @@ class EvoGraphOptimiser(GraphOptimiser):
                                    max_depth=self.max_depth, log=self.log) for new_ind in new_inds])
         for ind in new_inds:
             ind.fitness = None
+
         return new_inds
 
     def _make_population(self, pop_size: int) -> List[Any]:
@@ -382,16 +400,6 @@ class EvoGraphOptimiser(GraphOptimiser):
             num_of_new_individuals = self.requirements.pop_size
         return num_of_new_individuals
 
-    def default_on_next_iteration_callback(self, individuals, archive):
-        try:
-            self.history.add_to_history(individuals)
-            self.history.save_current_results()
-            archive = deepcopy(archive)
-            if archive is not None:
-                self.history.add_to_archive_history(archive.items)
-        except Exception as ex:
-            self.log.warn(f'Callback was not successful because of {ex}')
-
     def result_individual(self) -> Union[Any, List[Any]]:
         if not self.parameters.multi_objective:
             best = self.best_individual
@@ -399,11 +407,13 @@ class EvoGraphOptimiser(GraphOptimiser):
             best = self.archive.items
         return best
 
-    def _evaluate_individuals(self, individuals_set, objective_function, timer=None):
+    def _evaluate_individuals(self, individuals_set, objective_function, timer=None, n_jobs=1):
         evaluated_individuals = evaluate_individuals(individuals_set=individuals_set,
                                                      objective_function=objective_function,
                                                      graph_generation_params=self.graph_generation_params,
-                                                     timer=timer, is_multi_objective=self.parameters.multi_objective)
+                                                     is_multi_objective=self.parameters.multi_objective,
+                                                     n_jobs=n_jobs,
+                                                     timer=timer)
         individuals_set = correct_if_has_nans(evaluated_individuals, self.log)
         return individuals_set
 
