@@ -1,4 +1,5 @@
 from typing import List, Optional
+from uuid import uuid4
 
 from fedot.core.optimisers.graph import OptGraph
 from fedot.core.optimisers.opt_history import ParentOperator
@@ -9,10 +10,11 @@ ERROR_PREFIX = 'Invalid graph configuration:'
 class Individual:
     def __init__(self, graph: 'OptGraph', fitness: List[float] = None,
                  parent_operators: List[ParentOperator] = None, computation_time: Optional[int] = None):
+        self.uid = str(uuid4())
         self.parent_operators = parent_operators if parent_operators is not None else []
         self.fitness = fitness
         self.computation_time = computation_time
         self.graph = graph
 
     def __eq__(self, other):
-        return self.graph == other.graph
+        return self.uid == other.uid
