@@ -32,7 +32,8 @@ class PipelineTuner(HyperoptTuner):
 
         is_need_to_maximize = _greater_is_better(target=input_data.target,
                                                  loss_function=loss_function,
-                                                 loss_params=loss_params)
+                                                 loss_params=loss_params,
+                                                 data_type=input_data.data_type)
         self.is_need_to_maximize = is_need_to_maximize
 
         # Check source metrics for data
