@@ -1,4 +1,4 @@
-from fedot.core.utils import ComparableEnum as Enum
+from fedot.core.utilities.data_structures import ComparableEnum as Enum
 
 
 class DataTypesEnum(Enum):
@@ -8,6 +8,9 @@ class DataTypesEnum(Enum):
     # One dimensional array - time series
     ts = 'time_series'
 
+    # Table with different variant of time-series for the same variable as columns (used for extending train sample)
+    multi_ts = 'multiple_time_series'
+
     # Table, where cells contains text
     text = 'text'
 
