@@ -1,7 +1,6 @@
 from typing import Any, Dict
 
 from fedot.core.dag.graph_node import GraphNode
-
 from . import any_to_json
 
 
@@ -15,9 +14,9 @@ def graph_node_to_json(obj: GraphNode) -> Dict[str, Any]:
         if k not in ['_operator', '_fitted_operation', '_node_data']
     }
     encoded['content']['name'] = str(encoded['content']['name'])
-    if encoded['nodes_from']:
-        encoded['nodes_from'] = [
-            node._serialization_id
-            for node in encoded['nodes_from']
+    if encoded['_nodes_from']:
+        encoded['_nodes_from'] = [
+            node.uid
+            for node in encoded['_nodes_from']
         ]
     return encoded
