@@ -26,7 +26,12 @@ def ts_metric_calculation(reference_data: Union[InputData, List[Tuple[InputData,
     :param cache: cache manager for fitted models
     :param log: object for logging
     """
+    # TODO add support for multiprocessing
+    if __name__ != '__main__':
+        cache = None
+
     log.debug(f'Pipeline {pipeline.root_node.descriptive_id} fit for cross validation started')
+
     try:
         evaluated_metrics = [[] for _ in range(len(metrics))]
         fold_id = 0
