@@ -17,7 +17,8 @@ def cv_tabular_predictions(pipeline, reference_data: InputData, cv_folds: int):
         predictions.extend(predicted_values)
         targets.extend(actual_values)
 
-    if train_data.num_classes <= 2:
+    # TODO refactor this logic for classification
+    if train_data.num_classes is not None and train_data.num_classes <= 2:
         predictions, targets = np.ravel(np.array(predictions)), np.ravel(np.array(targets))
 
     return predictions, targets
